@@ -1,3 +1,5 @@
+from collections.abc import Iterator
+
 from numpy import inf
 from abc import ABC, abstractmethod
 from enum import Enum
@@ -219,6 +221,9 @@ class TransportSpace(ABC):
         return type(self) == type(other) and self.asdict() == other.asdict()
 
 
+class RequestGenerator(Iterator): ...
+
+
 Stoplist = List[Stop]
 """A list of `.Stop` objects. Specifies completely the current position and future actions a vehicle will make."""
 
@@ -258,3 +263,48 @@ Dispatcher = Callable[
 """Defines the `Dispatcher` interface. Actual dispatchers are implemented in `.util.dispatchers`."""
 
 Location = Union[int, float, tuple[float]]
+
+
+class DistanceDistribution(ABC):
+    """
+    Abstract base class for specifying distance distributions to use, e.g., during
+    request generation.
+    """
+
+    def __init__(self, *args, **kwargs): ...
+
+    @abstractmethod
+    def sample(self) -> float:
+        """
+        Sample a distance from the distribution.
+        """
+        ...
+
+    @property
+    @abstractmethod
+    def mean(self) -> float:
+        """
+        Return the mean of the distribution.
+        """
+        ...
+
+    @property
+    @abstractmethod
+    def std(self) -> float:
+        """
+        Return the standard deviation of the distribution.
+        """
+        ...
+
+    @abstractmethod
+    def __repr__(self): ...
+
+    @abstractmethod
+    def __str__(self): ...
+
+    @abstractmethod
+    def asdict(self) -> dict[str, Any]:
+        """
+        Return a dictionary representation of the distribution.
+        """
+        ...
